@@ -62,8 +62,7 @@
 
 #include <stdint.h>
 
-#if defined(WIN32) && !defined(__MINGW32__)
-#define LIBVNCSERVER_WORDS_BIGENDIAN
+#if defined(WIN32)
 typedef int8_t rfbBool;
 #include <sys/timeb.h>
 #include <winsock2.h>
@@ -79,11 +78,9 @@ typedef int8_t rfbBool;
 #endif
 
 #if LIBVNCSERVER_HAVE_ENDIAN_H
-#define _XOPEN_SOURCE
+# define _XOPEN_SOURCE
 # include <endian.h>
-# if defined(__BYTE_ORDER) && __BYTE_ORDER == __BIG_ENDIAN
-#  define LIBVNCSERVER_WORDS_BIGENDIAN 1
-# elsif defined(_BYTE_ORDER) && BYTE_ORDER == BIG_ENDIAN
+# if BYTE_ORDER == BIG_ENDIAN
 #  define LIBVNCSERVER_WORDS_BIGENDIAN 1
 # endif
 #endif
@@ -94,20 +91,34 @@ typedef int8_t rfbBool;
 #endif
 
 #define rfbMax(a,b) (((a)>(b))?(a):(b))
-#if !defined(WIN32) || defined(__MINGW32__)
+#ifdef WIN32
+#define rfbSocket SOCKET
+#define RFB_INVALID_SOCKET INVALID_SOCKET
+#define _rfbCloseSocket closesocket
+#else
 #ifdef LIBVNCSERVER_HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 #ifdef LIBVNCSERVER_HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#define SOCKET int
+#define rfbSocket int
+#define SOCKET int /* LibVNCServer versions older than 0.9.13 defined this for non-Windows, so keep it here */
+#define RFB_INVALID_SOCKET (-1)
+#define _rfbCloseSocket close
 typedef int8_t rfbBool;
 #undef FALSE
 #define FALSE 0
 #undef TRUE
 #define TRUE -1
 #endif
+#define rfbCloseSocket(s)			\
+    {						\
+	if (s != RFB_INVALID_SOCKET) {		\
+	    _rfbCloseSocket(s);		\
+	    s = RFB_INVALID_SOCKET;		\
+	}					\
+    }
 
 typedef uint32_t rfbKeySym;
 typedef uint32_t rfbPixel;
@@ -291,6 +302,8 @@ typedef char rfbProtocolVersionMsg[13];	/* allow extra byte for null */
 #define rfbVeNCrypt 19
 #define rfbSASL 20
 #define rfbARD 30
+#define rfbUltraMSLogonI 0x70	/* UNIMPLEMENTED */
+#define rfbUltraMSLogonII 0x71
 #define rfbMSLogon 0xfffffffa
 
 #define rfbVeNCryptPlain 256
@@ -421,6 +434,9 @@ typedef struct {
 #define rfbPalmVNCSetScaleFactor 0xF
 /* Xvp message - bidirectional */
 #define rfbXvp 250
+/* SetDesktopSize client -> server message */
+#define rfbSetDesktopSize 251
+#define rfbQemuEvent 255
 
 
 
@@ -500,6 +516,7 @@ typedef struct {
 
 #define rfbEncodingLastRect           0xFFFFFF20
 #define rfbEncodingNewFBSize          0xFFFFFF21
+#define rfbEncodingExtDesktopSize     0xFFFFFECC
 
 #define rfbEncodingQualityLevel0   0xFFFFFFE0
 #define rfbEncodingQualityLevel1   0xFFFFFFE1
@@ -512,6 +529,8 @@ typedef struct {
 #define rfbEncodingQualityLevel8   0xFFFFFFE8
 #define rfbEncodingQualityLevel9   0xFFFFFFE9
 
+#define rfbEncodingQemuExtendedKeyEvent 0xFFFFFEFE /* -258 */
+#define rfbEncodingExtendedClipboard 0xC0A1E5CE
 
 /* LibVNCServer additions.   We claim 0xFFFE0000 - 0xFFFE00FF */
 #define rfbEncodingKeyboardLedState   0xFFFE0000
@@ -1135,6 +1154,64 @@ typedef struct {
 #define rfbXvp_Reboot 3
 #define rfbXvp_Reset 4
 
+/*-----------------------------------------------------------------------------
+ * ExtendedDesktopSize server -> client message
+ *
+ * Informs the client of (re)size of framebuffer, provides information about
+ * physical screens attached, and lets the client knows it can request
+ * resolution changes using SetDesktopSize.
+ */
+
+typedef struct rfbExtDesktopSizeMsg {
+    uint8_t numberOfScreens;
+    uint8_t pad[3];
+
+    /* Followed by rfbExtDesktopScreen[numberOfScreens] */
+} rfbExtDesktopSizeMsg;
+
+typedef struct rfbExtDesktopScreen {
+    uint32_t id;
+    uint16_t x;
+    uint16_t y;
+    uint16_t width;
+    uint16_t height;
+    uint32_t flags;
+} rfbExtDesktopScreen;
+
+#define sz_rfbExtDesktopSizeMsg (4)
+#define sz_rfbExtDesktopScreen (16)
+
+/* x - reason for the change */
+#define rfbExtDesktopSize_GenericChange 0
+#define rfbExtDesktopSize_ClientRequestedChange 1
+#define rfbExtDesktopSize_OtherClientRequestedChange 2
+
+/* y - status code for change */
+#define rfbExtDesktopSize_Success 0
+#define rfbExtDesktopSize_ResizeProhibited 1
+#define rfbExtDesktopSize_OutOfResources 2
+#define rfbExtDesktopSize_InvalidScreenLayout 3
+
+/*-----------------------------------------------------------------------------
+ * SetDesktopSize client -> server message
+ *
+ * Allows the client to request that the framebuffer and physical screen
+ * resolutions are changed.
+ */
+
+typedef struct rfbSetDesktopSizeMsg {
+    uint8_t type;                       /* always rfbSetDesktopSize */
+    uint8_t pad1;
+    uint16_t width;
+    uint16_t height;
+    uint8_t numberOfScreens;
+    uint8_t pad2;
+
+    /* Followed by rfbExtDesktopScreen[numberOfScreens] */
+} rfbSetDesktopSizeMsg;
+
+#define sz_rfbSetDesktopSizeMsg (8)
+
 
 /*-----------------------------------------------------------------------------
  * Modif sf@2002
@@ -1189,7 +1266,8 @@ typedef union {
 	rfbPalmVNCReSizeFrameBufferMsg prsfb; 
 	rfbFileTransferMsg ft;
 	rfbTextChatMsg tc;
-        rfbXvpMsg xvp;
+	rfbXvpMsg xvp;
+	rfbExtDesktopSizeMsg eds;
 } rfbServerToClientMsg;
 
 
@@ -1327,6 +1405,17 @@ typedef struct {
 #define sz_rfbKeyEventMsg 8
 
 
+typedef struct {
+    uint8_t type;     /* always rfbQemuEvent */
+    uint8_t subtype;  /* always 0 */
+    uint16_t down;
+    uint32_t keysym;  /* keysym is specified as an X keysym, may be 0 */
+    uint32_t keycode; /* keycode is specified as XT key code */
+} rfbQemuExtendedKeyEventMsg;
+
+#define sz_rfbQemuExtendedKeyEventMsg 12
+
+
 /*-----------------------------------------------------------------------------
  * PointerEvent - mouse/pen move and/or button press.
  */
@@ -1363,6 +1452,16 @@ typedef struct {
     /* followed by char text[length] */
 } rfbClientCutTextMsg;
 
+#define rfbExtendedClipboard_Text 1
+#define rfbExtendedClipboard_RTF 2
+#define rfbExtendedClipboard_HTML 4
+#define rfbExtendedClipboard_DIB 8
+#define rfbExtendedClipboard_Files 16
+#define rfbExtendedClipboard_Caps (1 << 24)
+#define rfbExtendedClipboard_Request (1 << 25)
+#define rfbExtendedClipboard_Peek (1 << 26)
+#define rfbExtendedClipboard_Notify (1 << 27)
+#define rfbExtendedClipboard_Provide (1 << 28)
 #define sz_rfbClientCutTextMsg 8
 
 
@@ -1445,7 +1544,8 @@ typedef union {
 	rfbFileTransferMsg ft;
 	rfbSetSWMsg sw;
 	rfbTextChatMsg tc;
-        rfbXvpMsg xvp;
+	rfbXvpMsg xvp;
+	rfbSetDesktopSizeMsg sdm;
 } rfbClientToServerMsg;
 
 /* 
